//---------------------------------------------------------------------------
// TestMortality.cpp
//---------------------------------------------------------------------------
#include "TestCompetitionMortality.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "BehaviorBase.h"
#include <fstream>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
//  TestMortality()
/////////////////////////////////////////////////////////////////////////////
void clTestCompetitionMortality::TestMortality() {
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  float fVal;
  int **p_iTreeCount;
  int i, j, iNumSpecies, iNumTypes;

  //Create the sim manager and load our file
  clSimManager *p_oSimManager = new clSimManager(3, 2, " ");
  //Change to WriteXMLFile2 to test one of the errors - timestep>1
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;
  iNumSpecies = p_oPop->GetNumberOfSpecies();
  iNumTypes = p_oPop->GetNumberOfTypes();

  //Set up our tree count array
  p_iTreeCount = new int*[iNumSpecies];
  for (i = 0; i < iNumSpecies; i++) {
    p_iTreeCount[i] = new int[iNumTypes];
    for (j = 0; j < iNumTypes; j++) {
      p_iTreeCount[i][j] = 0;
    }
  }

  //Adults of Species 1
  fVal = 0.5;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(1, 1, 0, clTreePopulation::adult, 20);
    //Put a value of 0.5 in the "lgm" data member
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Growth", p_oTree->GetSpecies(), p_oTree->GetType()), fVal);
  }

  //Initial conditions count
  p_oAllTrees = p_oPop->Find("all");
  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {
    p_iTreeCount[p_oTree->GetSpecies()][p_oTree->GetType()]++;
    p_oTree = p_oAllTrees->NextTree();
  }

  CPPUNIT_ASSERT(p_iTreeCount[0][clTreePopulation::adult] == 1000);

  //Run for one timestep - everything except the light and growth behavior
  for (i = 2; i < p_oSimManager->GetNumberOfBehaviors(); i++)
    p_oSimManager->GetBehaviorObject(i)->Action();

  CPPUNIT_ASSERT((p_iTreeCount[0][clTreePopulation::adult] >= 517) && (p_iTreeCount[0][clTreePopulation::adult] >= 563));

  //Cleanup
    for (i = 0; i < iNumSpecies; i++) delete[] p_iTreeCount[i];
      delete[] p_iTreeCount;

    delete p_oSimManager;
}



/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////

//Rasmus parameter file
char * clTestCompetitionMortality::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>15</timesteps>"
       << "<rt_timestep>0</rt_timestep>"
       << "<randomSeed>937465</randomSeed>"
       << "<yearsPerTimestep>1.0</yearsPerTimestep>"
       //<< "<yearsPerTimestep>5.0</yearsPerTimestep>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>0.0</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       //<< "<tr_species speciesName=\"Species_2\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       //<< "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">40.0</tr_chVal>"
      // << "<tr_chVal species=\"Species_2\">40.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>"
       //<< "<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       //<< "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>"
       //<< "<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
        << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>"
       //<< "<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       //<< "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>"
       //<< "<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>"
      // << "<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>Constant GLI</behaviorName>"
       << "<version>1.0</version>"
       //<< "<applyTo species=\"Species_1\" type=\"Seedling\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
      // << "<applyTo species=\"Species_2\" type=\"Sapling\" />"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Adult\" />"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>ncigrowth</behaviorName>"
       << "<version>2.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Sapling\" />"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>competitionmortality</behaviorName>"
       << "<version>1.0</version>"
       //<< "<applyTo species=\"Species_1\" type=\"Seedling\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Sapling\" />"
      // << "<applyTo species=\"Species_2\" type=\"Adult\" />"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2.0</version>"
       //<< "<applyTo species=\"Species_1\" type=\"Seedling\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Sapling\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Adult\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<lightOther>"
       << "<li_constGLI>100</li_constGLI>"
       << "</lightOther>"
       << "<growth>"

       << "<gr_nciMaxPotentialGrowth>"
       << "<gr_nmpgVal species=\"Species_1\">0.2</gr_nmpgVal>"
       << "</gr_nciMaxPotentialGrowth>"

       << "<gr_nciMaxCrowdingRadius>"
       << "<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>"
       << "</gr_nciMaxCrowdingRadius>"

       << "<gr_nciAlpha>"
       << "<gr_naVal species=\"Species_1\">0</gr_naVal>"
       << "</gr_nciAlpha>"

       << "<gr_nciBeta>"
       << "<gr_nbVal species=\"Species_1\">0</gr_nbVal>"
       << "</gr_nciBeta>"

       << "<gr_nciSizeSensNCI>"
       << "<gr_nssnVal species=\"Species_1\">0</gr_nssnVal>"
       << "</gr_nciSizeSensNCI>"

       << "<gr_nciCrowdingSlope>"
       << "<gr_ncslVal species=\"Species_1\">0</gr_ncslVal>"
       << "</gr_nciCrowdingSlope>"

       << "<gr_nciCrowdingSteepness>"
       << "<gr_ncstVal species=\"Species_1\">1.0</gr_ncstVal>"
       << "</gr_nciCrowdingSteepness>"

       << "<gr_nciNeighStormEffMedDmg>"
       << "<gr_nnsemdVal species=\"Species_1\">1.0</gr_nnsemdVal>"
       << "</gr_nciNeighStormEffMedDmg>"

       << "<gr_nciNeighStormEffFullDmg>"
       << "<gr_nnsefdVal species=\"Species_1\">1.0</gr_nnsefdVal>"
       << "</gr_nciNeighStormEffFullDmg>"

       << "<gr_nciMinNeighborDBH>"
       << "<gr_nmndVal species=\"Species_1\">0</gr_nmndVal>"
       << "</gr_nciMinNeighborDBH>"

       << "<gr_nciDbhDivisor>100.0</gr_nciDbhDivisor>"
       << "<gr_nciRelativizeWithNCIMax>0</gr_nciRelativizeWithNCIMax>"
       << "<gr_nciIncludeSnagsInNCI>0</gr_nciIncludeSnagsInNCI>"

       << "<gr_nciSizeEffectMode>"
       << "<gr_nsemVal species=\"Species_1\">25</gr_nsemVal>"
       << "</gr_nciSizeEffectMode>"

       << "<gr_nciSizeEffectVariance>"
       << "<gr_nsevVal species=\"Species_1\">1</gr_nsevVal>"
       << "</gr_nciSizeEffectVariance>"

       << "<gr_nciShadingCoefficient>"
       << "<gr_nscVal species=\"Species_1\">0</gr_nscVal>"
       << "</gr_nciShadingCoefficient>"

       << "<gr_nciShadingExponent>"
       << "<gr_nseVal species=\"Species_1\">1.0</gr_nseVal>"
       << "</gr_nciShadingExponent>"

       << "<gr_nciStormEffMedDmg>"
       << "<gr_nsemdVal species=\"Species_1\">1.0</gr_nsemdVal>"
       << "</gr_nciStormEffMedDmg>"

       << "<gr_nciStormEffFullDmg>"
       << "<gr_nsefdVal species=\"Species_1\">1.0</gr_nsefdVal>"
       << "</gr_nciStormEffFullDmg>"

       << "<gr_nciSpecies_1NeighborLambda>"
       << "<gr_nlVal species=\"Species_1\">0</gr_nlVal>"
       << "</gr_nciSpecies_1NeighborLambda>"

       << "</growth>"


       << "<mortality>"
       << "<mo_CompMort>"
       << "<mo_cmVal species=\"Species_1\">0.05</mo_cmVal>"
       //<< "<mo_cmVal species=\"Species_2\">1</mo_cmVal>"
       << "</mo_CompMort>"
       << "<mo_CompMortMax>"
       << "<mo_cmmVal species=\"Species_1\">1</mo_cmmVal>"
       //<< "<mo_cmmVal species=\"Species_2\">0.45</mo_cmmVal>"
       << "</mo_CompMortMax>"
       << "</mortality>"
       << "</paramFile>";
oOut.close();
return cFileString;
}


/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////

//Rasmus parameter file
char * clTestCompetitionMortality::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile2.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>15</timesteps>"
       << "<rt_timestep>0</rt_timestep>"
       << "<randomSeed>937465</randomSeed>"
       << "<yearsPerTimestep>5.0</yearsPerTimestep>"
       //<< "<yearsPerTimestep>5.0</yearsPerTimestep>"
       << "<plot_lenX>100.0</plot_lenX>"
       << "<plot_lenY>100.0</plot_lenY>"
       << "<plot_latitude>0.0</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       //<< "<tr_species speciesName=\"Species_2\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       //<< "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">40.0</tr_chVal>"
      // << "<tr_chVal species=\"Species_2\">40.0</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>"
       //<< "<tr_sacrVal species=\"Species_2\">0.7</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       //<< "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>"
       //<< "<tr_cdtdVal species=\"Species_2\">0.7</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
        << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>"
       //<< "<tr_sachVal species=\"Species_2\">0.7</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       //<< "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>"
       //<< "<tr_sohdVal species=\"Species_2\">0.03</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>"
      // << "<tr_soahVal species=\"Species_2\">0.7</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>Constant GLI</behaviorName>"
       << "<version>1.0</version>"
       //<< "<applyTo species=\"Species_1\" type=\"Seedling\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
      // << "<applyTo species=\"Species_2\" type=\"Sapling\" />"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Adult\" />"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>ncigrowth</behaviorName>"
       << "<version>2.0</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Sapling\" />"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>competitionmortality</behaviorName>"
       << "<version>1.0</version>"
       //<< "<applyTo species=\"Species_1\" type=\"Seedling\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Sapling\" />"
      // << "<applyTo species=\"Species_2\" type=\"Adult\" />"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2.0</version>"
       //<< "<applyTo species=\"Species_1\" type=\"Seedling\" />"
       //<< "<applyTo species=\"Species_1\" type=\"Sapling\" />"
       << "<applyTo species=\"Species_1\" type=\"Adult\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Sapling\" />"
       //<< "<applyTo species=\"Species_2\" type=\"Adult\" />"
       << "</behavior>"
       << "</behaviorList>"
       << "<lightOther>"
       << "<li_constGLI>100</li_constGLI>"
       << "</lightOther>"
       << "<growth>"

       << "<gr_nciMaxPotentialGrowth>"
       << "<gr_nmpgVal species=\"Species_1\">0.2</gr_nmpgVal>"
       << "</gr_nciMaxPotentialGrowth>"

       << "<gr_nciMaxCrowdingRadius>"
       << "<gr_nmcrVal species=\"Species_1\">10</gr_nmcrVal>"
       << "</gr_nciMaxCrowdingRadius>"

       << "<gr_nciAlpha>"
       << "<gr_naVal species=\"Species_1\">0</gr_naVal>"
       << "</gr_nciAlpha>"

       << "<gr_nciBeta>"
       << "<gr_nbVal species=\"Species_1\">0</gr_nbVal>"
       << "</gr_nciBeta>"

       << "<gr_nciSizeSensNCI>"
       << "<gr_nssnVal species=\"Species_1\">0</gr_nssnVal>"
       << "</gr_nciSizeSensNCI>"

       << "<gr_nciCrowdingSlope>"
       << "<gr_ncslVal species=\"Species_1\">0</gr_ncslVal>"
       << "</gr_nciCrowdingSlope>"

       << "<gr_nciCrowdingSteepness>"
       << "<gr_ncstVal species=\"Species_1\">1.0</gr_ncstVal>"
       << "</gr_nciCrowdingSteepness>"

       << "<gr_nciMinNeighborDBH>"
       << "<gr_nmndVal species=\"Species_1\">0</gr_nmndVal>"
       << "</gr_nciMinNeighborDBH>"

       << "<gr_nciNeighStormEffMedDmg>"
       << "<gr_nnsemdVal species=\"Species_1\">1.0</gr_nnsemdVal>"
       << "</gr_nciNeighStormEffMedDmg>"

       << "<gr_nciNeighStormEffFullDmg>"
       << "<gr_nnsefdVal species=\"Species_1\">1.0</gr_nnsefdVal>"
       << "</gr_nciNeighStormEffFullDmg>"

       << "<gr_nciDbhDivisor>100.0</gr_nciDbhDivisor>"
       << "<gr_nciRelativizeWithNCIMax>0</gr_nciRelativizeWithNCIMax>"
       << "<gr_nciIncludeSnagsInNCI>0</gr_nciIncludeSnagsInNCI>"

       << "<gr_nciSizeEffectMode>"
       << "<gr_nsemVal species=\"Species_1\">25</gr_nsemVal>"
       << "</gr_nciSizeEffectMode>"

       << "<gr_nciSizeEffectVariance>"
       << "<gr_nsevVal species=\"Species_1\">1</gr_nsevVal>"
       << "</gr_nciSizeEffectVariance>"

       << "<gr_nciShadingCoefficient>"
       << "<gr_nscVal species=\"Species_1\">0</gr_nscVal>"
       << "</gr_nciShadingCoefficient>"

       << "<gr_nciShadingExponent>"
       << "<gr_nseVal species=\"Species_1\">1.0</gr_nseVal>"
       << "</gr_nciShadingExponent>"

       << "<gr_nciStormEffMedDmg>"
       << "<gr_nsemdVal species=\"Species_1\">1.0</gr_nsemdVal>"
       << "</gr_nciStormEffMedDmg>"

       << "<gr_nciStormEffFullDmg>"
       << "<gr_nsefdVal species=\"Species_1\">1.0</gr_nsefdVal>"
       << "</gr_nciStormEffFullDmg>"

       << "<gr_nciSpecies_1NeighborLambda>"
       << "<gr_nlVal species=\"Species_1\">0</gr_nlVal>"
       << "</gr_nciSpecies_1NeighborLambda>"

       << "</growth>"


       << "<mortality>"
       << "<mo_CompMort>"
       << "<mo_cmVal species=\"Species_1\">0.05</mo_cmVal>"
       //<< "<mo_cmVal species=\"Species_2\">1</mo_cmVal>"
       << "</mo_CompMort>"
       << "<mo_CompMortMax>"
       << "<mo_cmmVal species=\"Species_1\">1</mo_cmmVal>"
       //<< "<mo_cmmVal species=\"Species_2\">0.45</mo_cmmVal>"
       << "</mo_CompMortMax>"
       << "</mortality>"
       << "</paramFile>";
oOut.close();


  return cFileString;
}

