//---------------------------------------------------------------------------
// TestExpResourceMortality
//---------------------------------------------------------------------------
#if !defined(TestExpResourceMortality_H)
  #define TestExpResourceMortality_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

/**
* Tests the clExpResourceMortality class.
* @author Lora E. Murphy
*/
class clTestExpResourceMortality : public CppUnit::TestFixture
{
  public:

  /**
  * Tests the growth-and-resource mortality class.
  *
  * First run: WriteXMLFile1() creates the parameter file.  The file has the
  * following characteristics:
  * <ul>
  * <li>Timestep length of 1 year.</li>
  * <li>4 species: Species 1, Species 2, Species 3, Species 4 (Species 2 isn't 
  * used).</li>
  * <li>Growth and resource mortality applied to saplings of species 1 and 3,
  * and adults of species 1 and 4.</li>
  * <li>The resource grid has specified values for 5 cells.</li>
  * </ul>
  * The function does the following:
  * <ol>
  * <li>Grid 1:  Resource = 0, 1000 saplings of species 1 all with a value of 0
  * in their "Growth" data member; 1000 each of saplings and adults of 
  * species 2.</li>
  * <li>Grid 2: Resource = 0.5, 1000 adults of Species 1 all with a value of
  * 0 in their "Growth" data member.</li>
  * <li>Grid 3: Resource = 0, 1000 saplings of Species 3 all with a value of 4 
  * in their "Growth" data member.</li>
  * <li>Grid 4: Resource = 1, 1000 saplings of Species 1 all with a value of 0
  * in their "Growth" data member.</li>
  * <li>Grid 5: Resource = 0.6, 1000 adults of Species 4 all with a value of
  * 25.1 in their "Growth" data member.</li>
  * <li>Grid 6: Resource = 3, 1000 saplings of Species 4 all with a value of
  * 10 in their "Growth" data member.</li>
  * <li>Grid 7: Resource = 5, 1000 adults of Species 3 all with a value of 14
  * in their "Growth" data member.</li>
  * <li> The model is run for 1 timestep.</li>
  * <li> The remaining trees of each tree type and species are counted.</li>
  * <li> The counts are compared to expected results, give or take 5% (or
  * greater tolerance at lower numbers).</li>
  * </ol>
  *
  * Second run:
  * WriteXMLFile2() creates the parameter file.  Everything is exactly as it
  * is in the first run except the timestep length is 3 years.
  */
  void TestExpResourceMortality();

  protected:

  /**
  * Writes a parameter file with the specifications given for
  * TestExpResourceMortality(), run 1.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with the specifications given for
  * TestExpResourceMortality(), run 2.
  * @return Filename written.
  */
  char * WriteXMLFile2();

};
//---------------------------------------------------------------------------
#endif // TestExpResourceMortality_H
