//---------------------------------------------------------------------------

#ifndef TestGrowthOrgH
#define TestGrowthOrgH

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clGrowthOrg object.  This specifically makes sure that auto-update
* diameter growth and separate diameter-height growth updating work correctly.
* The individual growth behaviors are tested elsewhere.
*/
class clTestGrowthOrg : public CppUnit::TestFixture {

public:

  /**
  * Combines the testing into one behavior.
  */
  void TestGrowthOrg();


  private:

  /**
  * Tests normal processing.  WriteXMLFile1() is used to create the file for
  * this run.  Then six trees are added - a seedling, sapling, and species of
  * each of Species 1 and Species 2.  Both the diameter and height of each
  * tree are set to known values.  Then growth is run.  The diameter and height
  * for each tree are checked against expected values.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Tests to make sure that precedence requests are properly met when diameter
  * and height incrementers need to go in different orders.  WriteXMLFile2()
  * creates the file.  There are four species with each possible combination
  * of possible precedence.  One tree of each species is created; growth is
  * run; and the results are checked.
  * @param p_oSimManager Sim manager object for doing the run.
  */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  WriteXMLErrorFile1() and WriteXMLErrorFile2() are
  * used to introduce runs with error conditions in growth setup.
  * @param p_oSimManager Sim manager object for doing the runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to direct testing.  This contains two species, with
  * either constant basal area or constant radial growth assigned, with both
  * auto-update and diameter and height increment setting done separately.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a parameter file to direct testing for normal processing run 2.
  * This contains four species, with each one being assigned a diameter-height
  * growth behavior combination that checks all possible precedence request
  * situations.
  * @return Filename written.
  */
  char* WriteXMLFile2();


  /**
  * Writes a parameter file to direct testing for an intentionally triggered
  * error.  This file contains a tree species/type combo that has a diameter-
  * only growth behavior without a corresponding height-only growth behavior.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes a parameter file to direct testing for an intentionally triggered
  * error.  This file contains a tree species/type combo that has a height-
  * only growth behavior without a corresponding diameter-only growth behavior.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

};
//---------------------------------------------------------------------------
#endif
