//---------------------------------------------------------------------------
// TestHarvest
//---------------------------------------------------------------------------
#if !defined(TestHarvest_H)
  #define TestHarvest_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;
class clTreePopulation;

/**
* Tests the clDisturbance class when it's in Harvest mode.
*/
class clTestHarvest : public CppUnit::TestFixture
{
  public:

  /**
  * Groups together all the other harvest testing functions into a comprehensive
  * harvest test.
  */
  void TestHarvest();

  protected:

  /**
  * For testing trees.
  */
  struct tree{float fX, /**<X coordinate*/
              fY; /**<Y coordinate*/
              int iSpecies; /**<Tree species*/
              bool bDead, /**<whether the tree should be dead*/
              bFound;  /**<whether it was found in the tree population*/
              };

  /**
  * Performs the first harvest run.  This run has the following characteristics:
  * Harvest #	Timestep	Cut Ranges	Species	Cut Type	Cut Amt (abs)	Grids
  * 1	        1	        0 - 300	        2, 4	% BA	        35	        Whole plot
  * 2	        3	        0 - 30, 40 - 80	5	Abs BA	        0.2, 0.2	Whole plot
  * 3	        3	        Min 0	        2, 8	Clear	        100%	        Whole plot
  * The harvest will be performed.  Then the remaining trees are checked to make
  * sure the proper ones were left alone.
  */
  void DoHarvestRun1(clSimManager *p_oSimManager);

  /**
  * Performs the first harvest run.  This run has the following characteristics:
  * Harvest #	Timestep	Cut Ranges	Species	Cut Type	Cut Amt (abs)	Grids
  * 1	        1	        15-45, 50-80, 	0, 1	%Dens	        50, 40, 30, 20	Whole plot
  *                             80 - 85, 90 - 99
  * The harvest will be performed.  Then the remaining trees are checked to make
  * sure the proper ones were left alone.
  */
  void DoHarvestRun2(clSimManager *p_oSimManager);

/**
  * Performs the first harvest run.  This run has the following characteristics:
  * Harvest #	Timestep	Cut Ranges	Species	Cut Type	Cut Amt (abs)	Grids
  * 1	        1	        15-45, 50-80, 	6, 7	Abs Den	        1	        Whole plot
  *                             80 - 85, 90 - 100
  * The harvest will be performed.  Then the remaining trees are checked to make
  * sure the proper ones were left alone.
  */
  void DoHarvestRun3(clSimManager *p_oSimManager);

/**
  * Performs the first harvest run.  This run has the following characteristics:
  * Harvest #	Timestep	Cut Ranges	Species	Cut Type	Cut Amt (abs)	Grids
  * 1	        2	        Min 0	        0, 1, 2	Gap	        100%	        Whole plot
  * 2	        2	        15-45, 50-80, 	4, 5	% BA	        50, 100, 30, 0	(0,0) - (5,9)
  *                             80-85, 90-100
  * 3	        3	        200-300, 50-80, 4	Abs BA	        50, 100, 30, 0	(6,6) - (6-24); (8,6) - (8-24)
  *                             80-85, 90-100
  * 4	        3	        0-10, 10-20, 	7	% Dens	        50, 100, 30, 0	(0,0) - (5,9)
  *                             20-30, 30-40
  * The harvest will be performed.  Then the remaining trees are checked to make
  * sure the proper ones were left alone.
  */
  void DoHarvestRun4(clSimManager *p_oSimManager);

/**
  * Performs the first harvest run.  This run has the following characteristics:
  * Harvest #	Timestep	Cut Ranges	Species	Cut Type	Cut Amt (abs)	Grids
  * 1	        1	        0-10, 10-20, 	0-8	Abs Den	        5, 100, 3, 0	(6,6) - (6-24); (8,0) - (24-12)
  *                             20-30, 30-40
  * The harvest will be performed.  Then the remaining trees are checked to make
  * sure the proper ones were left alone.
  */
  void DoHarvestRun5(clSimManager *p_oSimManager);

/**
  * Performs the first harvest run.  This run has the following characteristics:
  * Harvest #	Timestep	Cut Ranges	Species	Cut Type	Cut Amt (abs)	Grids
  * 1	        1	        Min 0	        3	Clear	        100%	        (6,6) - (6-24); (8,6) - (8-24)
  * 2	        2	        Min 0	        0	Gap	        100%	        (0,0) - (5,9)
  * 3	        3	        0 - 300	        1	% BA	        50	        (0,10)
  * 4	        3	        0 - 300	        2	Abs BA	        4.5	        (5,5)
  * 5	        3	        0 - 300	        4	%Dens	        50	        (10,23)
  * 6	        3	        0 - 300	        5	Abs Den	        2	        (1,1)
  * 7	        3	        Min 0	        6	Clear	        100%	        (20,0)
  * 8	        3	        Min 0	        7	Gap	        100%	        (1,7)
  * The harvest will be performed.  Then the remaining trees are checked to make
  * sure the proper ones were left alone.
  */
  void DoHarvestRun6(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to test disturbance for run 1.  This has a tree
  * map so an expected tree set is created.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file to test disturbance for run 2.  This has a tree
  * map so an expected tree set is created.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file to test disturbance for run 3.  This has a tree
  * map so an expected tree set is created.
  * @return Filename written.
  */
  char * WriteXMLFile3();

  /**
  * Writes a parameter file to test disturbance for run 4.  This has a tree
  * map so an expected tree set is created.
  * @return Filename written.
  */
  char * WriteXMLFile4();

  /**
  * Writes a parameter file to test disturbance for run 5.  This has a tree
  * map so an expected tree set is created.
  * @return Filename written.
  */
  char * WriteXMLFile5();

  /**
  * Writes a parameter file to test disturbance for run 6.  This has a tree
  * map so an expected tree set is created.
  * @return Filename written.
  */
  char * WriteXMLFile6();

  /**
  * Writes the tree map for harvest parameter files.
  * @param cFileName Filestream to write to.
  */
  void WriteTreeMap(char *cFileName);

  /**
  * Checks the results of the tree population against the expected results.
  */
  void CheckResults(clTreePopulation *p_oPop, int iNumTrees, tree *p_FakeTrees);

};
//---------------------------------------------------------------------------
#endif // TestHarvest_H
