//---------------------------------------------------------------------------
// TestStormKiller
//---------------------------------------------------------------------------
#if !defined(TestStormKiller_H)
  #define TestStormKiller_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;
class clTreePopulation;

/**
* Tests the clStormKiller class.
*/
class clTestStormKiller : public CppUnit::TestFixture {

  public:

  /**
  * Assembles all other testing functions into one test.
  */
  void TestStormKiller();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() writes the parameter file.
  * Aspects of the run:
  * <ul>
  * <li>Grid resolution of "Storm Damage" is default.</li>
  * <li>Run is 5 timesteps.</li>
  * <li>1 year per timestep.</li>
  * <li>Snags are present in the run.</li>
  * <li>The following trees will be added:
  * <ul>
  * <li>Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 0.</li>
  * <li>Grid (0,1): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 1000.</li>
  * <li>Grid (0,2): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 2000.</li>
  * <li>Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 0.</li>
  * <li>Grid (1,1): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 1000.</li>
  * <li>Grid (1,2): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 2000.</li>
  * <li>All grids get 100 trees of DBH 4, Species 1, and DBH 5, Species 2,
  * which are never damaged or killed.</li>
  * <li>All grids get 100 trees of DBH 20, Species 1, Snag, which are never
  * removed.</li>
  * </ul></li>
  * </ul>
  *
  * The run proceeds for 5 timesteps and the number of trees with each damage
  * counter is checked through time.
  *
  * @param p_oSimManager Sim Manager object to run our test run.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Tests normal processing.  WriteXMLFile2() writes the parameter file.
  * Aspects of the run:
  *  <ul>
  * <li>Grid resolution of "Storm Damage" is default.</li>
  * <li>Run is 3 timesteps.</li>
  * <li>3 year per timestep.</li>
  * <li>Snags are present in the run.</li>
  * <li>The following trees will be added:
  * <ul>
  * <li>Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 0.</li>
  * <li>Grid (0,1): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 1000.</li>
  * <li>Grid (0,2): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 2000.</li>
  * <li>Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 0.</li>
  * <li>Grid (1,1): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 1000.</li>
  * <li>Grid (1,2): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 2000.</li>
  * <li>All grids get 100 trees of DBH 4, Species 1, and DBH 5, Species 2,
  * which are never damaged or killed.</li>
  * <li>All grids get 100 trees of DBH 20, Species 1, Snag, which are never
  * removed.</li>
  * </ul></li>
  * </ul>
  *
  * The run proceeds for 3 timesteps and the number of trees with each damage
  * counter is checked through time.
  *
  * @param p_oSimManager Sim Manager object to run our test run.
  */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);

  /**
  * Tests normal processing.  WriteXMLFile3() writes the parameter file.
  * Aspects of the run:
  * <ul>
  * <li>Grid resolution of "Storm Damage" is default.</li>
  * <li>Run is 3 timesteps.</li>
  * <li>3 year per timestep.</li>
  * <li>No snags in the run.</li>
  * <li>The following trees will be added:
  * <ul>
  * <li>Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 0.</li>
  * <li>Grid (0,1): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 1000.</li>
  * <li>Grid (0,2): 1000 trees of Species 1 and 2, DBH 30.  Damage counter is
  * 2000.</li>
  * <li>Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 0.</li>
  * <li>Grid (1,1): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 1000.</li>
  * <li>Grid (1,2): 1000 trees of Species 1 and 2, DBH 8.  Damage counter is
  * 2000.</li>
  * <li>All grids get 100 trees of DBH 4, Species 1, and DBH 5, Species 2,
  * which are never damaged or killed.</li>
  * <li>All grids get 100 trees of DBH 20, Species 1, Snag, which are never
  * removed.</li>
  * </ul></li>
  * </ul>
  *
  * The run proceeds for 5 timesteps and the number of trees with each damage
  * counter is checked through time.
  *
  * @param p_oSimManager Sim Manager object to run our test run.
  */
  void TestNormalProcessingRun3(clSimManager *p_oSimManager);

  /**
  * Checks the expected results for timestep 1 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep1(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 2 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep2(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 3 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep3(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 4 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep4(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 5 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep5(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 1 of TestNormalProcessingRun3().
  * @param p_oPop Tree population object.
  */
  void TestRun3Timestep1(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 2 of TestNormalProcessingRun3().
  * @param p_oPop Tree population object.
  */
  void TestRun3Timestep2(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 3 of TestNormalProcessingRun3().
  * @param p_oPop Tree population object.
  */
  void TestRun3Timestep3(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 4 of TestNormalProcessingRun3().
  * @param p_oPop Tree population object.
  */
  void TestRun3Timestep4(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 5 of TestNormalProcessingRun3().
  * @param p_oPop Tree population object.
  */
  void TestRun3Timestep5(clTreePopulation *p_oPop);

  /**
  * Tests error processing.
  * @throws error if any of the errors are not caught, or an error other than
  * the intended error is thrown by a parameter file.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for TestNormalProcessing1().
  * @return File name of the freshly written parameter file.
  */
  char *WriteXMLFile1();

  /**
  * Writes the parameter file for TestNormalProcessing2().
  * @return File name of the freshly written parameter file.
  */
  char *WriteXMLFile2();

  /**
  * Writes the parameter file for TestNormalProcessing3().
  * @return File name of the freshly written parameter file.
  */
  char *WriteXMLFile3();

  /**
  * Writes an error file where the storm behavior is not also enabled.  (By
  * necessity the storm damage applier behavior is not enabled either,
  * otherwise it would catch the error.)
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error file where the storm killer behavior is applied
  * to seedlings.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes an error file where the storm damage applier behavior is not also
  * enabled.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes an error file where the proportion of fully damaged trees that tip
  * up is not a proportion.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile4();

  /**
  * Writes an error file where the max snag lifespan is negative.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile5();

  /**
  * Writes an error file where mortality behaviors aren't applied to all
  * storm trees.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile6();

  /**
  * Writes common portions of the test parameter files.  Starts with
  * &lt;randomSeed&gt; and ends with &lt;/allometry&gt;.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);

  /**
  * Writes a behavior list with snags.
  * @param oOut File stream to write to.
  */
  void WriteSnagBehaviorList(std::fstream &oOut);

  /**
  * Writes a behavior list with no snags.
  * @param oOut File stream to write to.
  */
  void WriteNoSnagBehaviorList(std::fstream &oOut);

  /**
  * Writes common storm parameters.  Starts with &lt;storm&gt; and ends with 
  * &lt;/storm&gt;.
  * @param oOut File stream to write to.
  */
  void WriteStormParameters(std::fstream &oOut);


};
//---------------------------------------------------------------------------
#endif //TestStormKiller_H
